/****************************************************************************/
/****************************************************************************/
/** \file plugin-interface.c
    \brief Interfaces to and from plugins for the rest of the system.

    Contains the code for the functions to access plugins from the main program
    loop, callbacks for use by plugins, and some common code for building and
    accessing the packet structure.
*/
/****************************************************************************/

#include "packet.h"
#include "plugin.h"
#include "policy.h"
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <dirent.h>
#include <unistd.h>
#include <dlfcn.h>
#include <stdio.h>
#include "memory.h"
#include "naming.h"

#ifndef DEBUG_PLUGIN_INT
#ifdef DEBUG
#define DEBUG_PLUGIN_INT DEBUG
#else
#define DEBUG_PLUGIN_INT 0
#endif
#endif

extern PLUGIN *inp_list;
extern PLUGIN *anp_list;
extern PLUGIN *protop_list;
extern PLUGIN *outp_list;
extern PLUGIN *all_plugins;

void appendProtocol(PACKET *p, PROTO_LL *proto){
  PROTO_LL *temp;
  if(DEBUG_PLUGIN_INT>=DEBUG_DO_ENTRY_SOME){
    printf("Enter appendProtocol(PACKET*p(%p), PROTO_LL*proto(%p))\n", p, proto);
  }

  if(!p) return;
  if(!p->proto){
    p->proto = proto;
  }
  else {
    temp = p->proto;
    while(temp->next){
      temp = temp->next;
    }
    temp->next = proto;
  }
  return;
}


/****************************************************************************/
/****************** from plugin.h *******************************************/
/****************************************************************************/

/****************************************************************************/
/**

*/
/****************************************************************************/
int loadPlugins(char *dir){
  DIR *d;
  struct dirent *dirent;
  char *plugname;
  char *sTemp;
  unsigned int buf_sz=512;
  unsigned int i;
  void *dlopened;
  PLUGIN *plug;
  int err;
  unsigned short s;

  if(DEBUG_PLUGIN_INT>=DEBUG_DO_ENTRY_SOME){
    printf("Enter loadPlugins(char*dir(%s))\n", dir);
  }

  d = opendir(dir);
  if(!d){ 
    printf("ERR: opendir failed with %s\n", strerror(errno));
    return 0;
  }
  
  plugname = (char*)malloc_safe(buf_sz+NAME_MAX+1);
  if(!plugname){
    printf("ERR: malloc failed!\n");
    closedir(d);
    return 0;
  }
  dirent = readdir(d);
  while(dirent){
    if(DEBUG_PLUGIN_INT>=DEBUG_DO_LOOP){
      printf("In while(dirent) loop. d_name=%s\n", dirent->d_name);
    }
    i = strlen(dirent->d_name);
    if(i>4){
      sTemp = dirent->d_name + i-4;
      if(!strcasecmp(sTemp, ".plg")){
	if(dirent->d_name[0]!='/'){
	  sTemp = getcwd(plugname, buf_sz);
	  while(!sTemp && errno==ERANGE){
	    buf_sz+=256;
	    plugname = (char*)realloc_safe(plugname, buf_sz+NAME_MAX+1);
	    if(!plugname){
	      printf("ERR: realloc failed\n");
	      closedir(d);
	      return 0;
	    }
	    sTemp = getcwd(plugname, buf_sz);
	  }
	  plugname=sTemp;
	  strcat(plugname, "/");
	  strcat(plugname, dirent->d_name);
	}
	else strcpy(plugname, dirent->d_name);
	if(DEBUG_PLUGIN_INT>=DEBUG_DO_LOOP){
	  printf("......... Got plugin %s\n", plugname);
	}
	
	dlopened = dlopen(plugname, RTLD_NOW | RTLD_GLOBAL);
	if(!dlopened){
	  printf("ERR: %s couldn't be dlopened\n", plugname);
	  dirent = readdir(d);
	  continue;
	}

	plug = (PLUGIN*)malloc(sizeof(PLUGIN));
	if(!plug){
	  printf("ERR: Malloc failed\n");
	  return 0;
	}
	memset(plug, 0, sizeof(PLUGIN));

	plug->pluginRegister = dlsym(dlopened, "pluginRegister");
	if(!plug->pluginRegister){
	  printf("ERR: Couldn't find register fn for %s\n", plugname);
	  dirent=readdir(d);
	  continue;
	}
	
	(*(plug->pluginRegister))(&(plug->inPlug), &(plug->anPlug), 
				  &(plug->protoPlug), &(plug->outPlug), 
				  &decode);
	
	plug->pluginInit = dlsym(dlopened, "pluginInit");
	if(!plug->pluginRegister){
	  printf("ERR: Couldn't find init fn for %s\n", plugname);
	  dirent=readdir(d);
	  free(plug);
	  continue;
	}
		
	plug->pluginCleanup = dlsym(dlopened, "pluginCleanup");
	if(!plug->pluginRegister){
	  printf("ERR: Couldn't find cleanup fn for %s\n", plugname);
	  dirent=readdir(d);
	  free(plug);
	  continue;
	}
	
	if(plug->inPlug){
	  plug->pluginGetPacket = dlsym(dlopened, "pluginGetPacket");
	  if(!plug->pluginGetPacket){
	    printf("ERR: Couldn't find getpacket fn for %s\n", plugname);
	    plug->inPlug = (INPLUG*)0;
	  }
	  else {
	    plug->iNext = inp_list;
	    inp_list = plug;
	  }
	}

	if(plug->outPlug){
	  plug->pluginOutput = dlsym(dlopened, "pluginOutput");
	  if(!plug->pluginOutput){
	    printf("ERR: Couldn't find output fn for %s\n", plugname);
	    plug->outPlug = (OUTPLUG*)0;
	  }
	  else {
	    plug->oNext = outp_list;
	    outp_list = plug;
	  }
	}

	if(plug->anPlug){
	  plug->pluginAnalyse = dlsym(dlopened, "pluginAnalyse");
	  if(!plug->pluginAnalyse){
	    printf("ERR: Couldn't find analyse fn for %s\n", plugname);
	    plug->anPlug = (ANPLUG*)0;
	  }
	  else {
	    plug->aNext = anp_list;
	    anp_list = plug;
	  }
	}

	if(plug->protoPlug){
	  err = 0;
	  for(s=0;s<plug->protoPlug->nProto;s++){
	    storeProtoMapping((plug->protoPlug->sProto)[s],
			      (plug->protoPlug->pProto)[s]);
	  }
	  plug->pluginDecode = dlsym(dlopened, "pluginDecode");
	  if(!plug->pluginDecode){
	    err = 1;
	    printf("ERR: Couldn't find decode fn for %s\n", plugname);
	  }
	  plug->pluginTest = dlsym(dlopened, "pluginTest");
	  if(!plug->pluginTest){
	    err = 1;
	    printf("ERR: Couldn't find test fn for %s\n", plugname);
	  }
	  plug->pluginPrint = dlsym(dlopened, "pluginPrint");
	  if(!plug->pluginPrint){
	    err = 1;
	    printf("ERR: Couldn't find print fn for %s\n", plugname);
	  }
	  plug->pluginMkTest = dlsym(dlopened, "pluginMkTest");
	  if(!plug->pluginMkTest){
	    err = 1;
	    printf("ERR: Couldn't find MkTest fn for %s\n", plugname);
	  }

	  if(err){
	    plug->protoPlug = (PROTOPLUG*)0;
	  }
	  else {
	    plug->pNext = anp_list;
	    anp_list = plug;
	  }
	}

	if(!plug->inPlug && !plug->anPlug && !plug->outPlug 
	   && !plug->protoPlug){
	  free(plug);
	}
	else {
	  plug->next = all_plugins;
	  all_plugins = plug;
	}
	
	dlclose(dlopened);
      }
    }
    dirent = readdir(d);
  }

  closedir(d);

  return 1;
}

/****************************************************************************/
/**

*/
/****************************************************************************/
int initPlugin(PLUGIN *plug, int ct, char **inits){
  if(DEBUG_PLUGIN_INT>=DEBUG_DO_ENTRY_SOME){
    printf("Enter initPlugin(PLUGIN*plug(%p), int ct(%d), char** inits(%p))\n", plug, ct, inits);
  }
  return 0;
}

/****************************************************************************/
/**

*/
/****************************************************************************/
int decode(PACKET *p, PROTO protocol){
  if(DEBUG_PLUGIN_INT>=DEBUG_DO_ENTRY_SOME){
    printf("Enter decode(PACKET*p(%p), PROTO protocol(%lx))\n", p, protocol);
  }
  return 0;
}

/****************************************************************************/
/**

*/
/****************************************************************************/
int testVar(PROTO proto, PACKET *p,
	    long offset, char varType, char testType, unsigned long val){
  if(DEBUG_PLUGIN_INT>=DEBUG_DO_ENTRY_SOME){
    printf("Enter appendProtocol(PROTO proto(%lx),PACKET*p(%p),long offset(%ld),char varType(%c),char testType(%c),ulong val(%lx))\n", proto, p, offset, varType, testType, val);
  }
  return 0;
}

/****************************************************************************/
/**

*/
/****************************************************************************/
int testMask(PROTO proto, PACKET *p, 
	     long offset, long len, BYTE *val, BYTE *mask, BOOL type){
  if(DEBUG_PLUGIN_INT>=DEBUG_DO_ENTRY_SOME){
    printf("Enter testMask(PROTO proto(%lx),PACKET*p(%p),long offset(%ld),long len(%ld),BYTE*val(%p),BYTE*mask(%p),BOOL type(%c))\n", proto, p, offset, len, val, mask, type);
  }
  return 0;
}

/****************************************************************************/
/**

*/
/****************************************************************************/
int testSpecial(PROTO proto, PACKET *p,
		char *test, char *val){
  if(DEBUG_PLUGIN_INT>=DEBUG_DO_ENTRY_SOME){
    printf("Enter testSpecial(PROTO proto(%lx),PACKET*p(%p),char*test(%s),char*val(%s))\n", proto, p, test, val);
  }
  return 0;
}

/****************************************************************************/
/**

*/
/****************************************************************************/
char *analyse(PROTO proto, PACKET *p, char *type, char *args){
  if(DEBUG_PLUGIN_INT>=DEBUG_DO_ENTRY_SOME){
    printf("Enter analyse(PROTO proto(%lx),PACKET*p(%p),char*type(%s),char*args(%s))\n", proto, p, type, args);
  }
 return (char*)0;
}

/****************************************************************************/
/**

*/
/****************************************************************************/
int printVar(PROTO proto, PACKET *p,
	     char *args, int(*printFn)(char *fmt, ...)){
  if(DEBUG_PLUGIN_INT>=DEBUG_DO_ENTRY_SOME){
    printf("Enter printVar(PROTO proto(%lx),PACKET*p(%p),char*args(%s),printFn(%p))\n", proto, p, args, printFn);
  }
  return 0;
}

/****************************************************************************/
/**

*/
/****************************************************************************/
int mkTest(PROTO proto, char *val, char *test, char *field, POLICY_TEST *pTest){
  if(DEBUG_PLUGIN_INT>=DEBUG_DO_ENTRY_SOME){
    printf("Enter mkTest(PROTO proto(%lx),char*val(%s),char*test(%s),char*field(%s),POLICY_TEST*pTest(%p))\n", proto, val, test, field, pTest);
  }
  return 0;
}

/****************************************************************************/
/**

*/
/****************************************************************************/
int cleanupPlugins(void){
  PLUGIN *p;

  if(DEBUG_PLUGIN_INT>=DEBUG_DO_ENTRY_SOME){
    printf("Enter cleanupPlugins(void)\n");
  }

  p = all_plugins;
  while(p){
    (*(p->pluginCleanup))(0);
    free_safe(p);
  }
  all_plugins = (PLUGIN*)0;
  inp_list = (PLUGIN*)0;
  anp_list = (PLUGIN*)0;
  protop_list = (PLUGIN*)0;
  outp_list = (PLUGIN*)0;

  return 1;
}
