/****************************************************************************/
/****************************************************************************/
/** \file policy-build.c
    \brief Code to build, dump, and kill the policy structures.

    Contains code for building the policy, dumping it to a FILE*, and deleting 
    it. Must be remembered that this code may be called from a different
    thread than the main program is running in.
*/
/****************************************************************************/

#include "policy-build.h"
#include "memory.h"
#include "protocols.h"
#include <stdio.h>

#ifndef DEBUG_POLICY_BUILD
#ifdef DEBUG
#define DEBUG_POLICY_BUILD DEBUG
#else
#define DEBUG_POLICY_BUILD 0
#endif
#endif

POLICY_HEAD *newPolicy(void){
  POLICY_HEAD *ret;
  if(DEBUG_POLICY_BUILD>=DEBUG_DO_ENTRY_SOME){
    printf("Enter newPolicy\n");
  }
  ret = (POLICY_HEAD*)malloc_safe(sizeof(POLICY_HEAD));
  if(ret){
    ret->top = (POLICY_PROTO*)0;
    ret->def = (POLICY_OUTPUT*)0;
    ret->inits = (POLICY_INITS*)0;
    ret->input = (POLICY_INPUT*)0;
  }
  else printf("ERR: malloc failed\n");

  return ret;
}

/* processing rules */
POLICY_TEST *createBlankTest(void){
  POLICY_TEST *ret;
  if(DEBUG_POLICY_BUILD>=DEBUG_DO_ENTRY_SOME){
    printf("Enter createBlankTest\n");
  }
  ret = (POLICY_TEST*)malloc_safe(sizeof(POLICY_TEST));
  if(ret){
      ret->test_type = POLICY_TEST_TYPE_MASKED;
      ret->p_num = PROTO_ANY_NUM;
      ret->p_name = strdup_safe(PROTO_ANY_NAME);
      ret->m_val = (BYTE*)0;
      ret->m_mask = (BYTE*)0;
      ret->m_offset = 0;
      ret->m_len = 0;
      ret->m_type = '=';
      ret->v_val = 0;
      ret->v_vType = POLICY_TEST_V_TYPE_U32;
      ret->v_tType = strdup_safe("==");
      ret->a_name = (char*)0;
      ret->a_args = (char*)0;
      ret->result = (POLICY_RESULT*)0;
  }
  else printf("ERR: malloc failed\n");

  return ret;
}

POLICY_TEST *createAnalysisTest(char *plug,char *arg, char*protoname, PROTO num){
  POLICY_TEST *ret;
  if(DEBUG_POLICY_BUILD>=DEBUG_DO_ENTRY_SOME){
    printf("Enter createAnalysisTest(char*plug(%s),char*arg(%s))\n",plug,arg);
  }
  ret = (POLICY_TEST*)malloc_safe(sizeof(POLICY_TEST));
  if(ret){
    ret->p_num = num;
    ret->p_name = strdup_safe(protoname);
    ret->test_type = POLICY_TEST_TYPE_ANALYSIS;
    ret->a_name = strdup_safe(plug);
    ret->a_args = strdup_safe(arg);
    ret->result = (POLICY_RESULT*)0;
  }
  else printf("ERR: malloc failed\n");

  return ret;
}

POLICY_TEST *createDefaultTest(void){
  return createBlankTest();
}

POLICY_TEST *attachTestToResult(POLICY_RESULT *r,POLICY_TEST *t){
  POLICY_OUTPUT *o;
  if(DEBUG_POLICY_BUILD>=DEBUG_DO_ENTRY_SOME){
    printf("Enter attachTestToResult(POLICY_RESULT*r(%p),POLICY_TEST*t(%p))\n",r,t);
  }
  if(!r || !t) return (POLICY_TEST*)0;
  o = addOutputToResult(r,(char*)0,(char*)0,(char)0);
  if(!o) return (POLICY_TEST*)0;
  return attachTestToOutput(o,t);
}

POLICY_TEST *attachTestToProto(POLICY_PROTO *p,POLICY_TEST *t){
  if(DEBUG_POLICY_BUILD>=DEBUG_DO_ENTRY_SOME){
    printf("Enter attachTestToProto(POLICY_PROTO*p(%p),POLICY_TEST*t(%p))\n",p,t);
  }
  if(!p || !t) return (POLICY_TEST*)0;
  /**********************************************************/
  if(p->test){
    return (POLICY_TEST*)0;
  }
  else p->test = t;

  return t;
}

POLICY_TEST *attachTestToOutput(POLICY_OUTPUT *o,POLICY_TEST *t){
  POLICY_OUTPUT *oo;
  if(DEBUG_POLICY_BUILD>=DEBUG_DO_ENTRY_SOME){
    printf("Enter attachTestToOutput(POLICY_OUTPUT*o(%p),POLICY_TEST*t(%p))\n",o,t);
  }
  if(!o || !t) return (POLICY_TEST*)0;
  oo = o;
  while(oo->tNext){
    if(!(oo->oNext)){
      oo->oNext = (POLICY_OUTPUT*)malloc_safe(sizeof(POLICY_OUTPUT));
      if(!(oo->oNext)){
	printf("ERR: malloc failed\n");
	return (POLICY_TEST*)0;
      }
      oo->oNext->name = (char*)0;
      oo->oNext->args = (char*)0;
      oo->oNext->oNext = (POLICY_OUTPUT*)0;
      oo->oNext->tNext = (POLICY_TEST*)0;
    }
    oo = oo->oNext;
  }
  oo->tNext = t;
  return t;
}

POLICY_TEST *attachTestToPolicy(POLICY_HEAD *h,POLICY_TEST *t){
  POLICY_PROTO *p;
  if(DEBUG_POLICY_BUILD>=DEBUG_DO_ENTRY_SOME){
    printf("Enter attachTestToPolicy(POLICY_HEAD*h(%p),POLICY_TEST*t(%p))\n",h,t);
  }
  if(!h || !t) return (POLICY_TEST*)0;
  p = addProtoToPolicy(h,t->p_num, t->p_name);
  return attachTestToProto(p,t);
}

POLICY_PROTO *addProtoToPolicy(POLICY_HEAD *h,PROTO num,char *name){
  POLICY_PROTO *p;
  if(DEBUG_POLICY_BUILD>=DEBUG_DO_ENTRY_SOME){
    printf("Enter addProtoToPolicy(POLICY_HEAD*h(%p),PROTO num(%lx),char*name(%s))\n",h,num,name);
  }

  if(h->top){
    p = h->top;
    while(p->next){
      if(p->proto_num == num){
	return p;
      }
      p = p->next;
    }
    if(p->proto_num == num){
      return p;
    }
    p->next = (POLICY_PROTO*)malloc_safe(sizeof(POLICY_PROTO));
    if(!p->next){
      printf("ERR: malloc failed\n");
      return (POLICY_PROTO*)0;
    }
    p = p->next;
  }
  else {
    h->top = (POLICY_PROTO*)malloc_safe(sizeof(POLICY_PROTO));
    if(!h->top){
      printf("ERR: malloc failed\n");
      return (POLICY_PROTO*)0;
    }
    p = h->top;
  }
  p->proto_num = num;
  p->proto_name = strdup_safe(name);
  p->test = (POLICY_TEST*)0;
  p->next = (POLICY_PROTO*)0;
  return p;
}

POLICY_RESULT *addResultToTest(POLICY_TEST *t,char *sRes, unsigned short iRes){
  POLICY_RESULT *r;
  if(DEBUG_POLICY_BUILD>=DEBUG_DO_ENTRY_SOME){
    printf("Enter addResultToTest(POLICY_TEST*t(%p),char*sRes(%s),ushort iRes(%hu))\n",t,sRes,iRes);
  }
  if(!t) return (POLICY_RESULT*)0;
  r = t->result;
  if(!r){
    r = (POLICY_RESULT*)malloc_safe(sizeof(POLICY_RESULT));
    if(!r){
      printf("ERR: malloc failed\n");
      return (POLICY_RESULT*)0;
    }
    if(sRes) r->sResult = strdup_safe(sRes);
    else r->sResult = (char*)0;
    r->iResult = iRes;
    r->next = (POLICY_RESULT*)0;
    r->match = (POLICY_OUTPUT*)0;
    return r;
  }
  while(r){
    if(r->iResult==iRes){
      if(r->sResult==sRes) return r;
      else if(r->sResult && sRes){
	if(!strcmp(sRes, r->sResult)) return r;
      }
    }
    if(!(r->next)){
      r->next = (POLICY_RESULT*)malloc_safe(sizeof(POLICY_RESULT));
      if(!(r->next)){
	printf("ERR: malloc failed\n");
	return (POLICY_RESULT*)0;
      }
      r = r->next;
      if(sRes) r->sResult = strdup_safe(sRes);
      else r->sResult = (char*)0;
      r->iResult = iRes;
      r->next = (POLICY_RESULT*)0;
      r->match = (POLICY_OUTPUT*)0;
      return r;
    }
    r = r->next;
  }

  return (POLICY_RESULT*)0;
}

POLICY_OUTPUT *addOutputToResult(POLICY_RESULT *r,char *name,char *arg, char eof){
  POLICY_OUTPUT *o;
  if(DEBUG_POLICY_BUILD>=DEBUG_DO_ENTRY_SOME){
    printf("Enter addOutputToResult(POLICY_RESULT*r(%p),char*name(%s),char*arg(%s))\n",r,name,arg);
  }
  if(!r) return (POLICY_OUTPUT*)0;
  if(r->match){
    o = r->match;
    while(o->oNext){
      o = o->oNext;
    }
    o->oNext = (POLICY_OUTPUT*)malloc_safe(sizeof(POLICY_OUTPUT));
    o = o->oNext;
  }
  else{
    r->match = (POLICY_OUTPUT*)malloc_safe(sizeof(POLICY_OUTPUT));
    o = r->match;
  }
  if(o){
    if(name) o->name = strdup_safe(name);
    else o->name = (char*)0;
    if(arg) o->args = strdup_safe(arg);
    else o->args = (char*)0;
    o->eof = eof;
    o->oNext = (POLICY_OUTPUT*)0;
    o->tNext = (POLICY_TEST*)0;
  }
  else {
    printf("ERR: malloc failed\n");
  }

  return o;
}
POLICY_OUTPUT *addOutputToOutput(POLICY_OUTPUT *oo,char *name,char *arg, char eof){
  POLICY_OUTPUT *o;
  if(DEBUG_POLICY_BUILD>=DEBUG_DO_ENTRY_SOME){
    printf("Enter addOutputToOutput(POLICY_OUTPUT*oo(%p),char*name(%s),char*arg(%s),char eof(%hhd))\n",oo,name,arg,eof);
  }
  if(!oo) return (POLICY_OUTPUT*)0;
 
  o = oo;
  while(o->oNext){
    o = o->oNext;
  }
  o->oNext = (POLICY_OUTPUT*)malloc_safe(sizeof(POLICY_OUTPUT));
  o = o->oNext;

  if(o){
    if(name) o->name = strdup(name);
    else o->name = (char*)0;
    if(arg) o->args = strdup_safe(arg);
    else o->args = (char*)0;
    o->eof = eof;
    o->oNext = (POLICY_OUTPUT*)0;
    o->tNext = (POLICY_TEST*)0;
  }
  else {
    printf("ERR: malloc failed\n");
  }

  return o;
}

/* input rules */
POLICY_HEAD *addInputToPolicy(POLICY_HEAD *h,char *name, unsigned short weight,char *sSrc, int iSrc){
  POLICY_INPUT *i;
  if(DEBUG_POLICY_BUILD>=DEBUG_DO_ENTRY_SOME){
    printf("Enter addInputToPolicy(POLICY_HEAD*h(%p),char*name(%s),ushort weight(%hu),char*sSrc(%s),int iSrc(%d))\n",h,name,weight,sSrc,iSrc);
  }
  if(!h || !name) return (POLICY_HEAD*)0;
  
  if(h->input){
    i = h->input;
    while(i->next){
      if(!strcmp(i->plugname, name) && i->weight==weight && i->iSrc==iSrc){
	if(sSrc==i->sSrc) return h;
	else if(sSrc && i->sSrc){
	  if(!strcmp(sSrc, i->sSrc)) return h;
	}
      }
      i = i->next;
    }
    if(!strcmp(i->plugname, name) && i->weight==weight && i->iSrc==iSrc){
      if(sSrc==i->sSrc) return h;
      else if(sSrc && i->sSrc){
	if(!strcmp(sSrc, i->sSrc)) return h;
      }
    }
    i->next = (POLICY_INPUT*)malloc_safe(sizeof(POLICY_INPUT));
    i = i->next;
  }
  else {
    h->input = (POLICY_INPUT*)malloc_safe(sizeof(POLICY_INPUT));
    i = h->input;
  }
  if(i){
    i->plugname = strdup_safe(name);
    i->weight = weight;
    i->sSrc = strdup_safe(sSrc);
    i->iSrc = iSrc;
    return h;
  }
  else {
    printf("ERR: malloc failed\n");
  }
  return 0;
}

/* inits */
POLICY_HEAD *addInitToPolicy(POLICY_HEAD *h,char *name,char *init){
  POLICY_INITS *i;
  unsigned int u;
  if(DEBUG_POLICY_BUILD>=DEBUG_DO_ENTRY_SOME){
    printf("Enter addInitToPolicy(POLICY_HEAD*h(%p),char*name(%s),char*init(%s)\n",h,name,init);
  }
  if(!h || !name || !init) return (POLICY_HEAD*)0;
  if(h->inits){
    i = h->inits;
    while(i->next){
      if(!strcmp(name, i->plugname)) break;
      i = i->next;
    }
    if(!strcmp(name, i->plugname)){
      for(u=0;u<i->string_count;u++){
	if((i->init_string)[u]){
	  if(!strcmp((i->init_string)[u],init)){
	    return h;
	  }
	}
      }
    }
    else {
      i->next = (POLICY_INITS*)malloc_safe(sizeof(POLICY_INITS));
      if(!i->next){
	printf("ERR: malloc failed\n");
	return (POLICY_HEAD*)0;
      }
      i = i->next;
      i->plugname = strdup_safe(name);
      i->space = 100;
      i->init_string = (char**)malloc_safe(sizeof(char*)*i->space);
      if(!i->init_string){
	printf("ERR: malloc failed\n");
	free_safe(i);
	return (POLICY_HEAD*)0;
      }
      i->string_count = 0;
    }
    if(i->space<=i->string_count){
      i->space+=100;
      i->init_string = (char**)realloc_safe(i->init_string, sizeof(char*)*i->space);
      if(!i->init_string){
	printf("ERR: malloc failed\n");
	return (POLICY_HEAD*)0;
      }
    }
    (i->init_string)[i->string_count] = strdup_safe(init);
    i->string_count++;
  }
  else {
    h->inits = (POLICY_INITS*)malloc_safe(sizeof(POLICY_INITS));
    if(!h->inits){
      printf("ERR: malloc failed\n");
      return (POLICY_HEAD*)0;
    }
    i = h->inits;
    i->plugname = strdup_safe(name);
    i->space = 100;
    i->init_string = (char**)malloc_safe(sizeof(char*)*i->space);
    if(!i->init_string){
      printf("ERR: malloc failed\n");
      free_safe(i);
      return (POLICY_HEAD*)0;
    }
    i->string_count = 1;
    (i->init_string)[0] = strdup_safe(init);
    return h;
  }
  return (POLICY_HEAD*)0;
}
