/****************************************************************************/
/****************************************************************************/
/** \file policy-xml.c
    \brief Code for building policy from xml

    The code is to handle an entire XML document either from file or in
    memory, parse it and create a rubicon policy from it.
*/
/****************************************************************************/

#include "policy-xml.h"
#include "policy-build.h"
#include "policy-dump.h"
#include "policy-snort.h"
#include "memory.h"
#include "plugin.h"
#include "naming.h"
#include <unistd.h>
#include <errno.h>

#ifndef DEBUG_POLICY_XML
#ifdef DEBUG
#define DEBUG_POLICY_XML DEBUG
#else
#define DEBUG_POLICY_XML 0
#endif
#endif

DEFINE_LL *defs;

/****************************************************************************/
/** \brief 
    \param arr (char*) String of hex to convert into array
    \returns Pointer to malloc'ed byte array
 */
/****************************************************************************/
BYTE *convertStringToBArray(char *arr){
  BYTE *s;
  unsigned long arr_sz;
  unsigned long i,j;
  BYTE c;
  if(DEBUG_POLICY_XML>=DEBUG_DO_ENTRY_SOME){
    printf("Enter convertStringToBArray(char*arr(%s))\n",arr);
  }
  if(!arr) return (BYTE*)0;
  
  arr_sz = strlen(arr);
  if(arr_sz%2){
    printf("ERR: Array size must be an even number of hex characters\n");
    return (BYTE*)0;
  }
  s = (BYTE*)malloc_safe(arr_sz/2);
  if(!s){
    printf("ERR: malloc failed\n");
    return (BYTE*)0;
  }
  for(j=0,i=0;j<arr_sz;j++){
    switch(arr[j]){
    case '0': c = 0x00; break;
    case '1': c = 0x01; break;
    case '2': c = 0x02; break;
    case '3': c = 0x03; break;
    case '4': c = 0x04; break;
    case '5': c = 0x05; break;
    case '6': c = 0x06; break;
    case '7': c = 0x07; break;
    case '8': c = 0x08; break;
    case '9': c = 0x09; break;
    case 'a':
    case 'A': c = 0x0a; break;
    case 'b':
    case 'B': c = 0x0b; break;
    case 'c':
    case 'C': c = 0x0c; break;
    case 'd':
    case 'D': c = 0x0d; break;
    case 'e':
    case 'E': c = 0x0e; break;
    case 'f':
    case 'F': c = 0x0f; break;
    default:
      c  = 0x00;
      printf("ERR: Character %c encountered. Only 1234567890aAbBcCdDeEfF allowed. Using 0\n",arr[j]);
    }
    if(j%2){
      s[i] |= c;
      i++;
    }
    else {
      s[i] = (c<<4) & 0xf0;
    }
  }
  

  return (BYTE*)s;
}

/****************************************************************************/
/** \brief 
    \param doc (xmlDocPtr) Pointer to XML document
    \param node (xmlNodePtr) Pointer to current xml node
    \param pol_hd (::POLICY_HEAD*) The policy to add to
    \param pol_o (::POLICY_OUTPUT*) The output to append to
 */
/****************************************************************************/
int buildXmlSnort(xmlDocPtr doc, xmlNodePtr node, POLICY_HEAD* pol_hd, POLICY_OUTPUT* pol_o){
  char *source;
  char *filename;
  char *rules;
  unsigned long rules_sz;
  FILE *fd;
  char *inline_rules;
  unsigned long got;

  if(DEBUG_POLICY_XML>=DEBUG_DO_ENTRY_SOME){
    printf("Enter buildXmlSnort(xmlDocPtr doc(%p),xmlNodePtr node(%p),POLICY_HEAD*pol_hd(%p),POLICY_OUTPUT*pol_o(%p))\n",doc,node,pol_hd,pol_o);
  }
  if(!doc || !node) return 0;

  source = xmlGetProp(node, (xmlChar*)"source");
  if(!source){
    printf("ERR: XML: Oops - you forgot to give the snort source\n");
    return 0;
  }

  if(!strcmp(source, "file")){
    filename = xmlGetProp(node, (xmlChar*)"filename");
    if(!filename){
      printf("ERR: XML: Duh, if you give file as source, you need to give a filename as well\n");
    }

    fd = fopen(filename, "r");
    if(!fd){
      printf("ERR: fopen of file=(%s) failed\n",filename);
      return 0;
    }
    if(fseek(fd, 0, SEEK_END)==-1){
      printf("ERR: File fd=(%p) filename=(%s) fseek failed\n",fd,filename);
      return 0;
    }
    rules_sz = ftell(fd);
    if(rules_sz<=0){
      printf("ERR: Snort file has size (%ld)\n",rules_sz);
      return 0;
    }
    rewind(fd);
    rules = (char*)malloc(rules_sz+1);
    if(!rules){
      printf("ERR: XML: Malloc failed\n");
      return 0;
    }
    got = fread(rules, 1, rules_sz, fd);
    if(got!=rules_sz){
      printf("ERR: fread returned different size to sz. Got %ld instead of %ld\n",got,rules_sz);
      free(rules);
      return 0;
    }
    fclose(fd);

    rules[rules_sz]=(char)0;
  }
  else {
      inline_rules = xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
      if(!inline_rules){
	printf("ERR: XML: Duh, if you specify inline snort rules - you need to actually include them!\n");
	return 0;
      }
      rules_sz = strlen(inline_rules);
      if(!rules_sz){
	printf("ERR: XML: Duh, if you specify inline snort rules - you need to actually include them!\n");
	return 0;
      }
      rules = (char*)malloc_safe(rules_sz+1);
      if(!rules){
	printf("ERR: XML: malloc failed\n");
	return 0;
      }
      inline_rules[rules_sz] = (char)0;
      strcpy(rules, inline_rules);
      
  }
  rules_sz+=1;

  return parseSnortRules(pol_hd, pol_o, rules, rules_sz);
}

/****************************************************************************/
/** \brief 
    \param doc (xmlDocPtr) Pointer to XML document
    \param node (xmlNodePtr) Pointer to current xml node
    \param pol_hd (::POLICY_HEAD*) The policy to add to
 */
/****************************************************************************/
int buildXmlDefault(xmlDocPtr doc, xmlNodePtr node, POLICY_HEAD* pol_hd){
  POLICY_OUTPUT *o;
  xmlNodePtr cur;
  if(DEBUG_POLICY_XML>=DEBUG_DO_ENTRY_SOME){
    printf("Enter buildXmlDefault(xmlDocPtr doc(%p), xmlNodePtr node(%p),POLICY_HEAD*pol_hd(%p)\n",doc,node,pol_hd);
  }

  if(!doc || !node || !pol_hd) return 0;

  if(!(pol_hd->def)){
    o = (POLICY_OUTPUT*)malloc_safe(sizeof(POLICY_OUTPUT));
    if(!o){
      printf("ERR: XML: malloc failed\n");
      return 0;
    }
    o->name = (char*)0;
    o->args = (char*)0;
    o->oNext = (POLICY_OUTPUT*)0;
    o->tNext = (POLICY_TEST*)0;
  }
  else o = pol_hd->def;
  
  cur = node->xmlChildrenNode;
  while(cur){
    if(!xmlStrcmp(cur->name, (xmlChar*)"output")){
      buildXmlOutput(doc, cur, o);
    }
    else printf("WARN: XML: Unknown or malplaced tag %s\n",cur->name);
    cur = cur->next;
  }

  return 0;
}

/****************************************************************************/
/** \brief 
    \param doc (xmlDocPtr) Pointer to XML document
    \param node (xmlNodePtr) Pointer to current xml node
    \param pol_o (::POLICY_OUTPUT*) The output to append to
 */
/****************************************************************************/
int buildXmlOutput(xmlDocPtr doc, xmlNodePtr node, POLICY_OUTPUT* pol_o){
  char *name;
  char *arg;
  char *sEof;
  char eof;
  POLICY_OUTPUT *new_o;
  if(DEBUG_POLICY_XML>=DEBUG_DO_ENTRY_SOME){
    printf("Enter buildXmlOutput(xmlDocPtr doc(%p),xmlNodePtr node(%p),POLICY_OUTPUT*pol_o(%p))\n",doc,node,pol_o);
  }
  if(!doc || !node || !pol_o) return 0;

  name = xmlGetProp(node, (xmlChar*)"name");
  if(!name){
    printf("ERR: XML: Oops - you forgot to name the plugin to init\n");
    return 0;
  }
  arg = xmlGetProp(node, (xmlChar*)"arg");
  sEof = xmlGetProp(node, (xmlChar*)"eof");

  eof = (char)0;
  if(sEof){
    if(!strcmp(sEof, "yes")){
      eof = (char)1;
    }
  }
  
  new_o = addOutputToOutput(pol_o,name,arg,eof);
  if(!new_o){
    printf("ERR: XML: Failed to add output to output\n");
    return 0;
  }

  return 1;
}

/****************************************************************************/
/** \brief 
    \param doc (xmlDocPtr) Pointer to XML document
    \param node (xmlNodePtr) Pointer to current xml node
 */
/****************************************************************************/
int buildXmlDefine(xmlDocPtr doc, xmlNodePtr node){
  DEFINE_LL *l;
  char *id;
  if(DEBUG_POLICY_XML>=DEBUG_DO_ENTRY_SOME){
    printf("Enter buildXmlDefine(xmlDocPtr doc(%p),xmlNodePtr node(%p)\n",doc,node);
  }
  if(!doc || !node) return 0;

  id = xmlGetProp(node, (xmlChar*)"id");
  if(!id){
    printf("ERR: XML: Idiot - you forgot the id for one of your defines\n");
    return 0;
  }

  l = defs;
  while(l){
    if(!strcmp(id, l->id)){
      printf("ERR: XML: Blind are we? Or stupid? You've used the same id twice\n");
      return 0;
    }
    if(!l->next){
      break;
    }
    l = l->next;
  }
  l->next = (DEFINE_LL*)malloc(sizeof(DEFINE_LL));
  if(!(l->next)){
    printf("ERR: XML: malloc failed\n");
    return 0;
  }
  l->id = id;
  l->doc = doc;
  l->node = node;
  l->next = (DEFINE_LL*)0;

  return 1;
}

/****************************************************************************/
/** \brief 
    \param doc (xmlDocPtr) Pointer to XML document
    \param node (xmlNodePtr) Pointer to current xml node
    \param pol_hd (::POLICY_HEAD*) The policy head
    \param pol_o (::POLICY_OUTPUT*) The output to append to
 */
/****************************************************************************/
int buildXmlProtocol(xmlDocPtr doc, xmlNodePtr node, POLICY_HEAD* pol_hd, POLICY_OUTPUT* pol_o){
  xmlNodePtr cur;
  char *name;
  char *type;
  char *val;
  char *mask;
  char *offset;
  char *invert;
  char *vartype;
  char *test;
  char *field;
  PROTO protonum;
  POLICY_TEST *t;

  if(DEBUG_POLICY_XML>=DEBUG_DO_ENTRY_SOME){
    printf("Enter \n");
  }

  t = createBlankTest();
  if(!t){
    printf("ERR: XML: Failed to create blank test in protocol\n");
    return 0;
  }

  name = xmlGetProp(node, (xmlChar*)"name");
  protonum = mapProtoNameToNum(name);
  if(!protonum){
    protonum = PROTO_ANY_NUM;
    name = PROTO_ANY_NAME;
  }
  type = xmlGetProp(node, (xmlChar*)"type");
  if(!type){
    type = "english";
  }
  t->p_num = protonum;
  t->p_name = strdup_safe(name);
  if(!strcmp(type, "masked")){
    t->test_type = POLICY_TEST_TYPE_MASKED;
    val = xmlGetProp(node, (xmlChar*)"val");
    mask = xmlGetProp(node, (xmlChar*)"mask");
    if(val || mask){
      if(strlen(mask)==strlen(val)){
	if(strlen(val)%2 || strlen(mask)%2){
	  printf("ERR: XML: The val and mask must be ana even number of hex characters\n");
	  free_safe(t->p_name);
	  free_safe(t);
	  return 0;
	}
	t->m_len = strlen(val)/2;
      }
      else {
	  printf("ERR: XML: The val and mask must be of the same length\n");
	  free_safe(t->p_name);
	  free_safe(t);
	  return 0;
	}
    }
    else t->m_len = 0;
    if(t->m_len){
      t->m_val = convertStringToBArray(val);
      t->m_mask = convertStringToBArray(mask);
      offset = xmlGetProp(node, (xmlChar*)"offset");
      if(!offset) t->m_offset = 0;
      else t->m_offset = strtoul(offset, (char**)0, 0);
      invert = xmlGetProp(node, (xmlChar*)"invert");
      if(!invert)
	t->m_type = '=';
      else if(!strcmp(invert, "yes"))
	t->m_type = '!';
      else 
	t->m_type = '=';
    }
    else {
      t->m_val = (BYTE*)0;
      t->m_mask = (BYTE*)0;
      t->m_offset = 0;
      t->m_type = '=';
    }
  }
  else if(!strcmp(type, "variable")){
    t->test_type = POLICY_TEST_TYPE_VAR;
    val = xmlGetProp(node, (xmlChar*)"val");
    t->v_val = strtoul(val, (char**)0, 0);
    vartype = xmlGetProp(node, (xmlChar*)"vartype");
    if(!vartype) vartype = "U8";
    else if(!strcmp(vartype, "8")) t->v_vType = POLICY_TEST_V_TYPE_8;
    else if(!strcmp(vartype, "16")) t->v_vType = POLICY_TEST_V_TYPE_16;
    else if(!strcmp(vartype, "U16")) t->v_vType = POLICY_TEST_V_TYPE_U16;
    else if(!strcmp(vartype, "32")) t->v_vType = POLICY_TEST_V_TYPE_32;
    else if(!strcmp(vartype, "U32")) t->v_vType = POLICY_TEST_V_TYPE_U32;
    else t->v_vType = POLICY_TEST_V_TYPE_U8;

    test = xmlGetProp(node, (xmlChar*)"test");
    if(!test) test = "==";
    if(!strcmp(test, ">") || !strcmp(test, " >") || !strcmp(test, "> ")){
      t->v_tType = strdup(">");
    }
    else if(!strcmp(test, "<") || !strcmp(test, " <") || !strcmp(test, "< ")){
      t->v_tType = strdup("<");
    }
    else if(!strcmp(test, ">=") || !strcmp(test, "=>")){
      t->v_tType = strdup(">=");
    }
    else if(!strcmp(test, "<=") || !strcmp(test, "=<")){
      t->v_tType = strdup("<=");
    }
    else if(!strcmp(test, "!") || !strcmp(test, "!=")){
      t->v_tType = strdup("!=");
    }
    else {
      t->v_tType = strdup("==");
    }

    offset = xmlGetProp(node, (xmlChar*)"offset");
    t->v_offset = strtoul(offset, (char**)0, 0);
  }
  else if(!strcmp(type, "english")){
    val = xmlGetProp(node, (xmlChar*)"val");
    test = xmlGetProp(node, (xmlChar*)"test");
    field = xmlGetProp(node, (xmlChar*)"field");
    if(!mkTest(protonum, val, test, field, t)){
      printf("ERR: Failed to create english test\n");
      free_safe(t);
      return 0;
    }
  }

  if(pol_hd){
    if(!attachTestToPolicy(pol_hd, t)){
      printf("ERR: XML: Failed to attachTestToPolicy\n");
      free_safe(t);
      return 0;
    }
  }
  else {
    if(!attachTestToOutput(pol_o, t)){
      printf("ERR: XML: Failed to attachTestToOutput\n");
      free_safe(t);
      return 0;
    }
  }
  
  t->result = addResultToTest(t, (char*)0, 1);
  if(!(t->result)){
    printf("ERR: XML: Failed to addResultToTest\n");
    free_safe(t);
    return 0;
  }

  t->result->match = addOutputToResult(t->result, (char*)0, (char*)0, 0);
  if(!(t->result->match)){
    printf("ERR: XML: Failed to addOutputToResult\n");
    free_safe(t->result);
    free_safe(t);
    return 0;
  }

  cur = node->xmlChildrenNode;
  while(cur){
    if(!xmlStrcmp(cur->name, (xmlChar*)"analysis")){
      buildXmlAnalysis(doc, cur, t->result->match);
    }
    else if(!xmlStrcmp(cur->name, (xmlChar*)"output")){
      buildXmlOutput(doc, cur, t->result->match);
    }
    else if(!xmlStrcmp(cur->name, (xmlChar*)"protocol")){
      buildXmlProtocol(doc, cur, (POLICY_HEAD*)0, t->result->match);
    }
    else if(!xmlStrcmp(cur->name, (xmlChar*)"instance")){
      buildXmlInstance(doc, cur, (POLICY_HEAD*)0, t->result->match);
    }
    else printf("WARN: XML: Unknown or misplaced tag %s\n",cur->name);
    cur = cur->next;
  }
  
  return 0;
  }

/****************************************************************************/
/** \brief 
    \param doc (xmlDocPtr) Pointer to XML document
    \param node (xmlNodePtr) Pointer to current xml node
    \param pol_o (::POLICY_OUTPUT*) The output to append to
 */
/****************************************************************************/
int buildXmlAnalysis(xmlDocPtr doc, xmlNodePtr node, POLICY_OUTPUT* pol_o){
  POLICY_TEST *t;
  PROTO num;
  char *name;
  char *arg;
  char *sProto;
  if(DEBUG_POLICY_XML>=DEBUG_DO_ENTRY_SOME){
    printf("Enter buildXmlAnalysis(xmlDocPtr doc(%p),xmlNodePtr node(%p),POLICY_OUTPUT*pol_o(%p))\n",doc,node,pol_o);
  }
  if(!doc || !node || !pol_o) return 0;

  name = xmlGetProp(node, (xmlChar*)"name");
  if(!name){
    printf("ERR: XML: Oops, name missing\n");
    return 0;
  }
  arg = xmlGetProp(node, (xmlChar*)"arg");
  sProto = xmlGetProp(node, (xmlChar*)"protocol");
  if(sProto){
    num = mapProtoNameToNum(sProto);
    if(!num) printf("ERR: XML: Unknown proto %s. Using any as proto.\n",sProto);
  }
  else num = 0;
  if(!num){
    num = PROTO_ANY_NUM;
    sProto = PROTO_ANY_NAME;
  }

  t = createAnalysisTest(name,arg,sProto,num);
  if(!t){
    printf("ERR: XML: Failed to create analysis test\n");
    return 0;
  }
  if(!attachTestToOutput(pol_o,t)){
    printf("ERR: XML: Failed to attach analysis to test\n");
    dumpTest(0, t, 0, (char)1);
    return 0;
  }

  return 0;
}

/****************************************************************************/
/** \brief 
    \param doc (xmlDocPtr) Pointer to XML document
    \param node (xmlNodePtr) Pointer to current xml node
    \param pol_h (::POLICY_HEAD*) Pointer to head of policy
    \param pol_o (::POLICY_OUTPUT*) The output to append to
 */
/****************************************************************************/
int buildXmlInstance(xmlDocPtr doc, xmlNodePtr node, POLICY_HEAD *pol_h, POLICY_OUTPUT* pol_o){
  DEFINE_LL *l;
  char *id;
  xmlNodePtr cur;
  if(DEBUG_POLICY_XML>=DEBUG_DO_ENTRY_SOME){
    printf("Enter buildXmlInstance(xmlDocPtr doc(%p), xmlNodePtr node(%p), POLICY_HEAD*pol_h(%p),POLIC_OUTPUT*pol_o(%p))\n",doc, node,pol_h,pol_o);
  }
  if(!doc || !node) return 0;

  id = xmlGetProp(node, (xmlChar*)"id");
  if(!id){
    printf("ERR: XML: Doh! You forgot to say which defines to use (i.e. id)\n");
    return 0;
  }

  l = defs;
  while(l){
    if(strcmp(id, l->id)) break;
    l = l->next;
  }
  if(!l){
    printf("ERR: XML: Unknown id %s\n",id);
    return 0;
  }
  
  cur = l->node->xmlChildrenNode;
  while(cur){
    if(!xmlStrcmp(cur->name, (xmlChar*)"analysis")){
      if(!pol_o){
	printf("ERR: XML: Can't instantiate a define containing analysis as first layer\n");
      }
      else 
	buildXmlAnalysis(doc, cur, pol_o);
    }
    else if(!xmlStrcmp(cur->name, (xmlChar*)"protocol")){
      buildXmlProtocol(doc, cur, pol_h, pol_o);
    }
    else if(!xmlStrcmp(cur->name, (xmlChar*)"output")){
      if(!pol_o){
	printf("ERR: XML: Can't place output on first layer\n");
      }
      else 
	buildXmlOutput(doc, cur, pol_o);
    }
    else if(!xmlStrcmp(cur->name, (xmlChar*)"instance")){
      buildXmlInstance(doc, cur, pol_h, pol_o);
    }
    else printf("WARN: XML: Unknown or malplaced tag %s\n",cur->name);
    cur = cur->next;
  }
  

  return 1;
}

/****************************************************************************/
/** \brief 
    \param doc (xmlDocPtr) Pointer to XML document
    \param node (xmlNodePtr) Pointer to current xml node
    \param pol_t (::POLICY_TEST*) The analysis type test to respond to
 */
/****************************************************************************/
int buildXmlAnalysisResult(xmlDocPtr doc, xmlNodePtr node, POLICY_TEST* pol_t){
  char *sRes;
  POLICY_OUTPUT *o;
  POLICY_RESULT *r;
  xmlNodePtr cur;
  if(DEBUG_POLICY_XML>=DEBUG_DO_ENTRY_SOME){
    printf("Enter buildXmlAnalysisResult(xmlDocPtr doc(%p),xmlNodePtr node(%p), POLICY_TEST*pol_t(%p))\n",doc,node,pol_t);
  }
  if(!doc || !node || !pol_t) return 0;
  if(pol_t->test_type != POLICY_TEST_TYPE_ANALYSIS){
    printf("ERR: XML: Policy test must be of type analysis for an analysisResult\n");
    return 0;
  }

  sRes = xmlGetProp(node, (xmlChar*)"match");
  if(!sRes){
    sRes = "";
  }
  
  r = addResultToTest(pol_t,sRes,0);
  if(!r){
    printf("ERR: XML: Failed to addResultToTest\n");
    return 0;
  }

  o = addOutputToResult(r,(char*)0,(char*)0,0);
  if(!o){
    printf("ERR: XML: Failed to add default output to analysisResult\n");
    return 0;
  }

  cur = node->xmlChildrenNode;
  while(cur){
    if(!xmlStrcmp(cur->name, (xmlChar*)"analysis")){
      buildXmlAnalysis(doc, cur, o);
    }
    else if(!xmlStrcmp(cur->name, (xmlChar*)"protocol")){
      buildXmlProtocol(doc, cur, (POLICY_HEAD*)0, o);
    }
    else if(!xmlStrcmp(cur->name, (xmlChar*)"output")){
      buildXmlOutput(doc, cur, o);
    }
    else if(!xmlStrcmp(cur->name, (xmlChar*)"instance")){
      buildXmlInstance(doc, cur, (POLICY_HEAD*)0, o);
    }
    else printf("WARN: XML: Unknown or malplaced tag %s\n",cur->name);
    cur = cur->next;
  }
  

  return 1;
}

/****************************************************************************/
/** \brief 
    \param mem (char*) Pointer to the memory holding the policy
    \param sz (unsigned long) Size of mem
 */
/****************************************************************************/
POLICY_HEAD *buildXmlPolicy(char *mem, unsigned long sz){
  POLICY_HEAD *pol;
  xmlDocPtr doc;
  xmlNodePtr cur;
  char *version;
  DEFINE_LL *l;

  if(!mem) return (POLICY_HEAD*)0;
  if(DEBUG_POLICY_XML>=DEBUG_DO_ENTRY_SOME){
    printf("Enter buildXmlPolicy(char*mem(%p),ulong sz(%lu))\n", mem, sz);
  }

  defs = (DEFINE_LL*)0;

  LIBXML_TEST_VERSION
    xmlKeepBlanksDefault(0);

  doc = xmlParseMemory(mem, sz);
  if(!doc){
    printf("ERR: XML: Parsing failed\n");
    return (POLICY_HEAD*)0;
  }

  cur = xmlDocGetRootElement(doc);
  if(!doc){
    printf("ERR: XML: Failed to get root element\n");
    return (POLICY_HEAD*)0;
  }

  if(xmlStrcmp(cur->name, (xmlChar*)"rubicon")){
    xmlFreeDoc(doc);
    printf("ERR: XML: Unknown document - root element must be rubicon\n");
    return (POLICY_HEAD*)0;
  }
  version = xmlGetProp(cur, (xmlChar*)"version");
  if(!version){
    xmlFreeDoc(doc);
    printf("ERR: XML: Version number missing\n");
    return (POLICY_HEAD*)0;
  }
  else if(xmlStrcmp(version, (xmlChar*)"1.1")){
    xmlFreeDoc(doc);
    printf("ERR: XML: Sorry, only v 1.1 not v %s supported\n", version);
    return (POLICY_HEAD*)0;
  }

  cur = cur->xmlChildrenNode;
  if(xmlStrcmp(cur->name, (xmlChar*)"policy")){
    xmlFreeDoc(doc);
    printf("ERR: XML: Next node after rubicon must be policy\n");
    return (POLICY_HEAD*)0;
  }

  pol = newPolicy();
  if(!pol){
    xmlFreeDoc(doc);
    return (POLICY_HEAD*)0;
  }

  cur = cur->xmlChildrenNode;
  while(cur){
    if(!xmlStrcmp(cur->name, (xmlChar*)"init")){
      buildXmlInit(doc, cur, pol);
    }
    else if(!xmlStrcmp(cur->name, (xmlChar*)"input")){
      buildXmlInput(doc, cur, pol);
    }
    else if(!xmlStrcmp(cur->name, (xmlChar*)"default")){
      buildXmlDefault(doc, cur, pol);
    }
    else if(!xmlStrcmp(cur->name, (xmlChar*)"protocol")){
      buildXmlProtocol(doc, cur, pol, (POLICY_OUTPUT*)0);
    }
    else if(!xmlStrcmp(cur->name, (xmlChar*)"define")){
      buildXmlDefine(doc, cur);
    }
    else if(!xmlStrcmp(cur->name, (xmlChar*)"instance")){
      buildXmlInstance(doc, cur, pol, (POLICY_OUTPUT*)0);
    }
    else if(!xmlStrcmp(cur->name, (xmlChar*)"snort")){
      buildXmlSnort(doc, cur, pol, (POLICY_OUTPUT*)0);
    }
    else printf("WARN: XML: Unknown tag %s\n",cur->name);
    cur = cur->next;
  }
  
  while(defs){
    l = defs;
    defs = defs->next;
    free(l);
  }

  xmlFreeDoc(doc);

  return pol;
}

/****************************************************************************/
/** \brief Load a policy XML file into memory then call buildXmlPolicy
    \param filename (char*) Name of file to load and parse
 */
/****************************************************************************/
POLICY_HEAD *buildXmlPolicyFromFile(char *filename){
  long sz;
  long got;
  unsigned long filename_len;
  FILE *fd;
  char *mem;
  char *fullfile;

  if(DEBUG_POLICY_XML>=DEBUG_DO_ENTRY_SOME){
    printf("Enter buildXmlPolicyFromFile(char*filename(%s))\n", filename);
  }

  if(!filename) return (POLICY_HEAD*)0;

  /* should convert to full path */
  if(filename[0]=='/'){
    fullfile = filename;
  }
  else {
    sz = 256;
    filename_len = strlen(filename) + 1;
    fullfile = (char*)malloc_safe(sz+filename_len);
    if(!fullfile){
      printf("ERR: malloc_safe failed\n");
      return (POLICY_HEAD*)0;
    }
    while(!getcwd(fullfile, sz) && errno==ERANGE){
      sz+=256;
      fullfile = (char*)realloc_safe(fullfile, sz+filename_len);
      if(!fullfile){
	printf("ERR: realloc failed\n");
	return (POLICY_HEAD*)0;
      }
    }
    strcat(fullfile, "/");
    strcat(fullfile, filename);
  }

  if(DEBUG_POLICY_XML>=DEBUG_DO_INTERNALS){
    printf("Full filename=%s\n", fullfile);
  }

  fd = fopen(fullfile, "r");
  if(!fd){
    printf("ERR: fopen of file=(%s) failed\n",fullfile);
    return (POLICY_HEAD*)0;
  }
  if(fseek(fd, 0, SEEK_END)==-1){
    printf("ERR: File fd=(%p) filename=(%s) fseek failed\n",fd,fullfile);
    return (POLICY_HEAD*)0;
  }
  sz = ftell(fd);
  if(sz<=0){
    printf("ERR: Policy file has size (%ld)\n",sz);
    return (POLICY_HEAD*)0;
  }
  rewind(fd);
  mem = (char*)malloc(sz+1);
  if(!mem){
    printf("ERR: Malloc failed\n");
    return (POLICY_HEAD*)0;
  }
  got = fread(mem, 1, sz, fd);
  if(got!=sz){
    printf("ERR: fread returned different size to sz. Got %ld instead of %ld\n",got,sz);
    free(mem);
    return (POLICY_HEAD*)0;
  }
  fclose(fd);

  mem[sz]=(char)0;
  return buildXmlPolicy(mem, sz);
}

/****************************************************************************/
/** \brief 
    \param doc (xmlDocPtr) Pointer to XML document
    \param node (xmlNodePtr) Pointer to current xml node
    \param pol_hd (::POLICY_HEAD*) Policy to set inits for
 */
/****************************************************************************/
int buildXmlInit(xmlDocPtr doc, xmlNodePtr node, POLICY_HEAD* pol_hd){
  char *name;
  char *init;
  if(DEBUG_POLICY_XML>=DEBUG_DO_ENTRY_SOME){
    printf("Enter buildXmlInit(xmlDocPtr doc(%p),xmlNodePtr node(%p),POLICY_HEAD*pol_hd(%p))\n",doc,node,pol_hd);
  }
  if(!doc || !node || !pol_hd) return 0;

  name = xmlGetProp(node, (xmlChar*)"name");
  if(!name){
    printf("ERR: XML: Oops - you forgot to name the plugin to init\n");
  }
  init = xmlNodeListGetString(doc, node->xmlChildrenNode, 1);

  if(!addInitToPolicy(pol_hd,name,init)){
    printf("ERR: XML: Failed to add init %s to policy\n", name);
    return 0;
  }
  return 1;
}

/****************************************************************************/
/** \brief 
    \param doc (xmlDocPtr) Pointer to XML document
    \param node (xmlNodePtr) Pointer to current xml node
    \param pol_hd (::POLICY_HEAD*) Policy to set input for
 */
/****************************************************************************/
int buildXmlInput(xmlDocPtr doc, xmlNodePtr node, POLICY_HEAD* pol_hd){
  char *name;
  char *sWeight;
  unsigned short weight;
  char *sSrc;
  if(DEBUG_POLICY_XML>=DEBUG_DO_ENTRY_SOME){
    printf("Enter buildXmlInput(xmlDocPtr doc(%p),xmlDocPtr node(%p),POLICY_HEAD*pol_hd(%p))\n",doc,node,pol_hd);
  }

  name = xmlGetProp(node, (xmlChar*)"name");
  if(!name){
    printf("ERR: XML: Oops - you forgot to name the plugin to init\n");
  }
  sWeight = xmlGetProp(node, (xmlChar*)"weight");
  sSrc = xmlGetProp(node, (xmlChar*)"src");

  if(!sWeight) weight=1;
  else {
    weight = (unsigned short)strtoul(sWeight, (char**)0, 0);
  }

  if(!addInputToPolicy(pol_hd,name,weight,sSrc,0)){
    printf("ERR: XML: Failed to add input %s to policy\n", name);
    return 0;
  }
  return 1;
}
