/****************************************************************************/
/****************************************************************************/
/** \file rubicon.c
    \brief The main rubicon program loop

*/
/****************************************************************************/

#include "plugin.h"
#include "policy.h"
#include "types.h"
#include "policy-xml.h"
#include "policy-dump.h"
#include "naming.h"
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>

PLUGIN *inp_list=0;
PLUGIN *anp_list=0;
PLUGIN *protop_list=0;
PLUGIN *outp_list=0;
PLUGIN *all_plugins=0;

int cleanup=0;
int shutdown=0;


#ifndef DEBUG_RUBICON
#ifdef DEBUG
#define DEBUG_RUBICON DEBUG
#else
#define DEBUG_RUBICON 0
#endif
#endif

/****************************************************************************/
/** \brief Signal handler
    \param i (int) Signal received.
    
    If a SIGHUP is received, then the program reinitialises as soon as it is
    safe to do so. If a SIGINT is received, the handler marks the program
    to cleanup when safe, and then shutdown. If a second SIGINT is received,
    the program shuts down immediately.
 */
/****************************************************************************/
void sig_handler(int i){
  printf("Got signal int=%d\n",i);
  switch(i){
  case SIGHUP:
    printf("HUP received. Flagged to reinitialise asap.\n");
    cleanup=1;
    break;
  case SIGINT:
    if(!shutdown){
      printf("INT received. Flagged to cleanup and then shutdown.\n");
      printf("Sending a second INT signal (e.g. kill -INT or control-c)\n");
      printf("if you want to immediately shutdown.\n");
      shutdown = 1;
    }
    else {
      printf("Second INT received. Shutting down now!\n");
      exit(0);
    }
    break;
  default:
  }
}

/****************************************************************************/
/** \brief Main. What else can you say? 
    \param argc (int) Count of number of arguments
    \param argv (char *[]) Array of string arguments
 */
/****************************************************************************/
int main(int argc, char *argv[]){
  POLICY_HEAD *pol;
  char *plug_dir;
  char *log_location;
  char *in_file;
  char *config_file;
  char c;
  struct sigaction sigact;

  /* set defaults */
  plug_dir = ".";
  log_location = "./log/";
  in_file = (char*)0;
  config_file = (char*)0;

  /* get arguments */
  while((c=getopt(argc, argv, "p:h?l:i:c:d:"))!=(char)-1){
    switch(c){
    case 'l':
      log_location = optarg;
      break;
    case 'i':
      in_file = optarg;
      break;
    case 'c': 
      config_file = optarg;
      break;
    case 'd':
      /** \todo store mapping */
      break;
    case 'p':
      plug_dir = optarg;
      break;
    case 'h':
    case '?':
    default:
      printf("Usage: ./rubicon [options]\n");
      printf("Options:-\n");
      printf("        -p <dir> Specifies the directory to look for plugins\n");
      printf("        -i <file> Input data file (pcap/tcpdump format)\n");
      printf("        -l <dir or file> Log directory/file. If dir, must exist.\n");
      printf("        -c <file> Config file\n");
      printf("        -d <dev>=<name>  Device name mapping. e.g. /dev/eth0=IN\n");
      printf("        -h or -?  Show this usage\n");
      exit(0);
    }
  }

  sigact.sa_handler = &sig_handler;
  sigact.sa_flags = 0;
  sigemptyset(&(sigact.sa_mask));
  if(sigaction(SIGINT, &sigact, (struct sigaction *)0)==-1){
    printf("WARN: sigaction SIGINT failed. errno=%d =%s\n", errno, strerror(errno));
  }

  if(sigaction(SIGHUP, &sigact, (struct sigaction *)0)==-1){
    printf("WARN: sigaction SIGHUP failed. errno=%d =%s\n", errno, strerror(errno));
  }

  
  if(!loadPlugins(plug_dir)){
    printf("ERR: loadPlugins failed\n");
    return 0;
  }

  /* start up the policy thread */
  pol = buildXmlPolicyFromFile(config_file);
  if(!pol){
    printf("ERR: The xml policy couldn't be loaded.\n");
    return 0;
  }
  
  dumpPolicy(stdout, pol, (char)0);

  /* start message loop */

  while(1){
    if(cleanup){
      cleanup = 0;
      cleanupPlugins();
      cleanMapping();
      if(DEBUG_RUBICON>=DEBUG_DO_INTERNALS) 
	dumpPolicy(stdout, pol, (char)1);
      else 
	dumpPolicy(0, pol, (char)1);
      
      pol = (POLICY_HEAD*)0;

      if(!loadPlugins(plug_dir)){
	printf("ERR: loadPlugins failed\n");
	return 0;
      }
      
      /* start up the policy thread */
      pol = buildXmlPolicyFromFile(config_file);
      if(!pol){
	printf("ERR: The xml policy couldn't be loaded.\n");
	return 0;
      }
    }
    if(shutdown){
      shutdown = 0;
      cleanup = 0;
      cleanupPlugins();
      cleanMapping();
      if(DEBUG_RUBICON>=DEBUG_DO_INTERNALS) 
	dumpPolicy(stdout, pol, (char)1);
      else 
	dumpPolicy(0, pol, (char)1);
      
      pol = (POLICY_HEAD*)0;
      exit(0);
    }
    /* new policy */
    /* -- if so, load, init */

    /* find source for packet */
    /* get packet */

    /* handle packet */
  }

  return 0;
}
