/*****************************************************************************/
/*****************************************************************************/
/** \file rubicon_snort.c
    \brief Do snort-style signature-based IDS test
*/
/*****************************************************************************/

#include "../../main/anplug.h"
#include "../../main/packet.h"
#include "../../main/memory.h"

#ifndef DEBUG_FILE
#ifdef DEBUG
#define DEBUG_FILE DEBUG
#else
#define DEBUG_FILE 0
#endif
#endif

/*! Pointer to generic protocol decoding function */
int (*Decode)(PACKET*, PROTO);

ANPLUG *ap = (ANPLUG*)0;

/****************************************************************************/
/** \brief Registration function
    \param inp(::INPLUG**) Placeholder for input plugin descriptor
    \param anp(::ANPLUG**) Placeholder for analysis plugin descriptor
    \param protop(::PROTOPLUG**) Placeholder for protocol plugin descriptor
    \param outp(::OUTPLUG**) Placeholder for output plugin descriptor
    \param DecodeCallback (function ptr) Pointer to generic decode function

    This function is called by the main process in order to register the
    plugin. The function then populates the placeholders for the plugin
    types it supports, and sets all others to null. The DecodeCallback function
    is stored in case the plugin ever wants to decode anything.
 */
/****************************************************************************/
int pluginRegister(INPLUG **inp, ANPLUG **anp, PROTOPLUG **protop, OUTPLUG **outp,
		   int(*DecodeCallback)(PACKET*,PROTO)){
  if(DEBUG_FILE>=DEBUG_DO_ENTRY_ALL){
    printf("Enter pluginRegister(inp(%lx),anp(%lx),protop(%lx),outp(%lx),DecodeCallback(%lx))\n", inp, anp, protop, outp, DecodeCallback);
  }

  Decode = DecodeCallback;
  *inp = (INPLUG*)0;
/*   *anp = &ap; */
  *outp = (OUTPLUG*)0;
  *protop = (PROTOPLUG*)0;

  if(!ap){
    ap = (ANPLUG*)malloc(sizeof(ANPLUG));
    if(!ap){
      printf("ERR: plugin register for ANPLUG failed\n");
      ap = (ANPLUG*)0;
    }
    else {
      ap->name = strdup_safe("SNORT");
      ap->version = 0x00010000;
    }
  }
  *anp = ap;

  return 0;
}

/****************************************************************************/
/** \brief Initialise the plugin
    \param ct (int) Number of initialisation strings
    \param inits (char **) Array of ct initialisation strings
 */
/****************************************************************************/
int pluginInit(int ct, char **inits){
  if(DEBUG_FILE>=DEBUG_DO_ENTRY_ALL){
    printf("Enter pluginInit(int ct(%d),char**inits(%lx))\n", ct, inits);
  }
  return 0;
}

/****************************************************************************/
/** \brief Cleanup the plugin
    \param reason (int) The reason for the cleanup.

    This call should always be made before closing the program, or
    reinitialising the plugin. It frees all the memory allocated and still held
    internally (i.e. it doesn't free memory that it has a pointer to but has
    passed elsewhere). Additionally, files are closed etc.
 */
/****************************************************************************/
int pluginCleanup(int reason){
  if(DEBUG_FILE>=DEBUG_DO_ENTRY_ALL){
    printf("Enter pluginCleanup(int reason(%d))\n", reason);
  }

  if(ap){
    free_safe(ap->name);
    free_safe(ap);
  }
  ap = (ANPLUG*)0;

  return 0;
}

/****************************************************************************/
/** \brief Analyses a packet
    \param p (::PACKET*) The packet to analyse
    \param anType (char*) The type of analysis to perform
    \param args (char*) Other invocation arguments
    \returns (char*) NULL on error, otherwise a string specifying the result

    This function performs some form of analysis on the packet, the type of
    which is given in anType. It returns a string describing the result. 
    Additionally, the function may append a 'special' protocol to the packet
    with further information. If this is done, the plugin should also 
    implement the protocol plugin interface to enable the 'special' protocol
    (i.e. the test results) to be tested.
*/
/****************************************************************************/
char *pluginAnalyse(PACKET *p, char *anType, char *args){
  if(DEBUG_FILE>=DEBUG_DO_ENTRY_ALL){
    printf("Enter pluginAnalyse(PACKET*p(%lx),char*anType(%s),char*args(%s))\n",p,anType,args);
  }

  return (char*)0;
}
