/****************************************************************************/
/****************************************************************************/
/** \file memory.h
    \brief Header for 'safe' memory stuff, e.g. wrappers for malloc
    \todo The list bit could be better done using a hash of some kind
    \todo Make thread safe
    \todo Remove counting when debug is 0
*/
/****************************************************************************/

#include "memory.h"
#include <limits.h>
#include <stdio.h>

void **memlist=0;
unsigned int count=0;
unsigned int sz=0;

/****************************************************************************/
/** \brief Append pointer to list
    \param ptr (void*) The pointer to append
    
    This function appends the pointer to the list, growing it if necessary.
 */
/****************************************************************************/
void addToList(void *ptr){
  if(sz<(count+1)){
    if(sz>(UINT_MAX-100)){
      printf("Woops: Overflow in memory.c sz\n");
    }
    sz += 100;
    memlist = (void**)realloc((void*)memlist, sz * sizeof(void*));
  }
  if(!memlist) return;
  memlist[count] = ptr;
  if(count==UINT_MAX){
    printf("Woops: Overflow in memory.c count\n");
  }
  count++;
}

/****************************************************************************/
/** \brief Removes pointer from list
    \param ptr (void *) Ptr to memory allocated from malloc (or similar)
    \returns 1 on success, 0 on fail (e.g. not found)

    This function removes the specified pointer from the array of pointers. 
    It then shrinks the array by 1 by replacing the removed pointer with the
    last pointer in the array.
 */
/****************************************************************************/
int removeFromList(void *ptr){
  unsigned int i=0;
  int found=0;

  if(!memlist) return 0;
  for(i=0;i<count;i++){
    if(memlist[i]==ptr){
      found=1;
      break;
    }
  }
  if(!found) return 0;
  count--;
  if(i<count){
    memlist[i] = memlist[count];
  }
  return 1;
}

/****************************************************************************/
/** \brief Wrapper for calloc
 */
/****************************************************************************/
void *calloc_safe(size_t nmemb, size_t size){
  void *temp;
  if(!nmemb || !size){
    printf("ERR: Calloc called with zero something. Nmemb=%x Size=%x.\n",nmemb, size);
    return (void*)0;
  }
  temp = calloc(nmemb, size);
  if(!temp){
    printf("WARN: Calloc returned null\n");
  }
  else addToList(temp);
  return temp;
}

/****************************************************************************/
/** \brief Wrapper for malloc
 */
/****************************************************************************/
void *malloc_safe(size_t size){
  void *temp;
  if(!size){
    printf("ERR: Calloc called with zero size\n");
    return (void*)0;
  }
  temp = malloc(size);
  if(!temp){
    printf("WARN: Malloc returned null\n");
  }
  else addToList(temp);
  return temp;
}

/****************************************************************************/
/** \brief Wrapper for free
 */
/****************************************************************************/
void free_safe(void *ptr){
  if(removeFromList(ptr)) free(ptr);
}

/****************************************************************************/
/** \brief Wrapper for realloc
 */
/****************************************************************************/
void *realloc_safe(void *ptr, size_t size){
  void *temp;
  if(!size){
    printf("ERR: Realloc called with zero something. ptr=%p Size=%x.\n",ptr, size);
    return (void*)0;
  }
  if(ptr && !removeFromList(ptr)){
    printf("ERR: Realloc tried to realloc a non-malloc sourced pointer(%p)\n",ptr);
    return (void*)0;
  }

  temp = (void*)realloc((void*)ptr, size);
  if(!temp){
    printf("WARN: Realloc returned null\n");
  }
  else addToList(temp);
  return temp;
}

/****************************************************************************/
/** \brief Wrapper for strdup
 */
/****************************************************************************/
char *strdup_safe(const char *s){
  char *temp;
  if(!s){
    printf("ERR: Strdup called with null string\n");
    return (void*)0;
  }
  temp = strdup(s);
  if(!temp){
    printf("WARN: Strdup returned null\n");
  }
  else addToList(temp);
  return temp;
}

/****************************************************************************/
/** \brief Frees all remaining pointers
    \returns number of outstanding pointers
 */
/****************************************************************************/
int cleanup_memory(void){
  unsigned int i;

  for(i=0;i<count;i++){
    if(memlist[i]) free(memlist[i]);
    else { printf("WARN: There was a null in memlist, count %u\n", i); }
  }

  return count;
}
