/*****************************************************************************/
/*****************************************************************************/
/** \file rubicon_ipq.c
    \brief Get packets from netfilter via libipq
*/
/*****************************************************************************/

#include "../../main/inplug.h"
#include "../../main/packet.h"
#include "../../main/memory.h"

#ifndef DEBUG_IPQ
#ifdef DEBUG
#define DEBUG_IPQ DEBUG
#else
#define DEBUG_IPQ 0
#endif
#endif

/*! Pointer to generic protocol decoding function */
int (*Decode)(PACKET*, PROTO);

INPLUG *ip = (INPLUG*)0;

/****************************************************************************/
/** \brief Registration function
    \param inp(::INPLUG**) Placeholder for input plugin descriptor
    \param anp(::ANPLUG**) Placeholder for analysis plugin descriptor
    \param protop(::PROTOPLUG**) Placeholder for protocol plugin descriptor
    \param outp(::OUTPLUG**) Placeholder for output plugin descriptor
    \param DecodeCallback (function ptr) Pointer to generic decode function

    This function is called by the main process in order to register the
    plugin. The function then populates the placeholders for the plugin
    types it supports, and sets all others to null. The DecodeCallback function
    is stored in case the plugin ever wants to decode anything.
 */
/****************************************************************************/
int pluginRegister(INPLUG **inp, ANPLUG **anp, PROTOPLUG **protop, OUTPLUG **outp,
		   int(*DecodeCallback)(PACKET*,PROTO)){
  if(DEBUG_IPQ>=DEBUG_DO_ENTRY_ALL){
    printf("Enter pluginRegister(inp(%lx),anp(%lx),protop(%lx),outp(%lx),DecodeCallback(%lx))\n", inp, anp, protop, outp, DecodeCallback);
  }

  Decode = DecodeCallback;
/*   *inp = &ip; */
  *anp = (ANPLUG*)0;
  *outp = (OUTPLUG*)0;
  *protop = (PROTOPLUG*)0;

  if(!ip){
    ip = (INPLUG*)malloc(sizeof(INPLUG));
    if(!ip){
      printf("ERR: plugin register for OUTPLUG failed\n");
      ip = (INPLUG*)0;
    }
    else {
      ip->name = strdup_safe("IPQ");
      ip->version = 0x00010000;
    }
  }
  *inp = ip;

  return 0;
}

/****************************************************************************/
/** \brief Initialise the plugin
    \param ct (int) Number of initialisation strings
    \param inits (char **) Array of ct initialisation strings
 */
/****************************************************************************/
int pluginInit(int ct, char **inits){
  if(DEBUG_IPQ>=DEBUG_DO_ENTRY_ALL){
    printf("Enter pluginInit(int ct(%d),char**inits(%lx))\n", ct, inits);
  }
  return 0;
}

/****************************************************************************/
/** \brief Cleanup the plugin
    \param reason (int) The reason for the cleanup.

    This call should always be made before closing the program, or
    reinitialising the plugin. It frees all the memory allocated and still held
    internally (i.e. it doesn't free memory that it has a pointer to but has
    passed elsewhere). Additionally, files are closed etc.
 */
/****************************************************************************/
int pluginCleanup(int reason){
  if(DEBUG_IPQ>=DEBUG_DO_ENTRY_ALL){
    printf("Enter pluginCleanup(int reason(%d))\n", reason);
  }

  if(ip){
    free_safe(ip->name);
    free_safe(ip);
  }
  ip = (INPLUG*)0;

  return 0;
}

/****************************************************************************/
/** \brief Gets a packet
    \param iSrc (int) Integer representing which source to select from
    \param sSrc (char*) String stating which source to get the packet from
    \param p (::PACKET**) Placeholder for the received packet.
    \returns (int) 1 on success, 0 on failure (due to timeout), -1 on failure due to an error

    This function retrieves a packet from whatever source. It must not block.
    The iSrc variable takes precedence over the sSrc when selecting the input
    source. If iSrc should be ignored, it's value must be set to -1.
*/
/****************************************************************************/
int pluginGetPacket(int iSrc, char *sSrc, PACKET **p){
  if(DEBUG_IPQ>=DEBUG_DO_ENTRY_ALL){
    printf("Enter pluginGetPacket(int iSrc(%d),char*sSrc(%s),PACKET**p(%lx))\n",iSrc, sSrc, p);
  }
  return 0;
}
