/*****************************************************************************/
/*****************************************************************************/
/** \file rubicon_mangle.c
    \brief Modify packet contents and payloads (NB doesn't actually output)
*/
/*****************************************************************************/

#include "../../main/outplug.h"
#include "../../main/packet.h"
#include "../../main/memory.h"

#ifndef DEBUG_MANGLE
#ifdef DEBUG
#define DEBUG_MANGLE DEBUG
#else
#define DEBUG_MANGLE 0
#endif
#endif

/*! Pointer to generic protocol decoding function */
int (*Decode)(PACKET*, PROTO);

OUTPLUG *op = (OUTPLUG*)0;

/****************************************************************************/
/** \brief Registration function
    \param inp(::INPLUG**) Placeholder for input plugin descriptor
    \param anp(::ANPLUG**) Placeholder for analysis plugin descriptor
    \param protop(::PROTOPLUG**) Placeholder for protocol plugin descriptor
    \param outp(::OUTPLUG**) Placeholder for output plugin descriptor
    \param DecodeCallback (function ptr) Pointer to generic decode function

    This function is called by the main process in order to register the
    plugin. The function then populates the placeholders for the plugin
    types it supports, and sets all others to null. The DecodeCallback function
    is stored in case the plugin ever wants to decode anything.
 */
/****************************************************************************/
int pluginRegister(INPLUG **inp, ANPLUG **anp, PROTOPLUG **protop, OUTPLUG **outp,
		   int(*DecodeCallback)(PACKET*,PROTO)){
  if(DEBUG_MANGLE>=DEBUG_DO_ENTRY_ALL){
    printf("Enter pluginRegister(inp(%lx),anp(%lx),protop(%lx),outp(%lx),DecodeCallback(%lx))\n", inp, anp, protop, outp, DecodeCallback);
  }

  Decode = DecodeCallback;
  *inp = (INPLUG*)0;
  *anp = (ANPLUG*)0;
/*   *outp = &op; */
  *protop = (PROTOPLUG*)0;

  if(!op){
    op = (OUTPLUG*)malloc(sizeof(OUTPLUG));
    if(!op){
      printf("ERR: plugin register for OUTPLUG failed\n");
      op = (OUTPLUG*)0;
    }
    else {
      op->name = strdup_safe("MANGLE");
      op->version = 0x00010000;
    }
  }
  *outp = op;

  return 0;
}

/****************************************************************************/
/** \brief Initialise the plugin
    \param ct (int) Number of initialisation strings
    \param inits (char **) Array of ct initialisation strings
 */
/****************************************************************************/
int pluginInit(int ct, char **inits){
  if(DEBUG_MANGLE>=DEBUG_DO_ENTRY_ALL){
    printf("Enter pluginInit(int ct(%d),char**inits(%lx))\n", ct, inits);
  }
  return 0;
}

/****************************************************************************/
/** \brief Cleanup the plugin
    \param reason (int) The reason for the cleanup.

    This call should always be made before closing the program, or
    reinitialising the plugin. It frees all the memory allocated and still held
    internally (i.e. it doesn't free memory that it has a pointer to but has
    passed elsewhere). Additionally, files are closed etc.
 */
/****************************************************************************/
int pluginCleanup(int reason){
  if(DEBUG_MANGLE>=DEBUG_DO_ENTRY_ALL){
    printf("Enter pluginCleanup(int reason(%d))\n", reason);
  }

  if(op){
    free_safe(op->name);
    free_safe(op);
  }
  op = (OUTPLUG*)0;

  return 0;
}

/****************************************************************************/
/** \brief Outputs something. Maybe.
    \param p (::PACKET*) The packet to output
    \param args (char*) Description of what to output

    The exact contents of the args differs widely between plugins. This plugin
    doesn't actually output anything. Instead, it messes around with the
    contents of the packet. Due to sequence number issues, it is normally
    no possible to shorten or lengthen the packet. The modified flag must be
    set on any modified protocol, so that checksums can be recalculated if 
    required.
*/
/****************************************************************************/
int pluginOutput(PACKET *p, char *args){
  if(DEBUG_MANGLE>=DEBUG_DO_ENTRY_ALL){
    printf("Enter pluginOutput(PACKET*p(%lx),char*args(%s))\n",p,args);
  }
  return 0;
}
