/*****************************************************************************/
/*****************************************************************************/
/** \file rubicon_ethernet.c
    \brief Ethernet protocol decoder
*/
/*****************************************************************************/

#include "../../main/protoplug.h"
#include "../../main/types.h"
#include "../../main/policy.h"
#include "../../main/memory.h"

#ifndef DEBUG_ETHERNET
#ifdef DEBUG
#define DEBUG_ETHERNET DEBUG
#else
#define DEBUG_ETHERNET 0
#endif
#endif

/*! Pointer to generic protocol decoding function */
int (*Decode)(PACKET*, PROTO);

PROTOPLUG *pp = (PROTOPLUG*)0;

/****************************************************************************/
/** \brief Registration function
    \param inp(::INPLUG**) Placeholder for input plugin descriptor
    \param anp(::ANPLUG**) Placeholder for analysis plugin descriptor
    \param protop(::PROTOPLUG**) Placeholder for protocol plugin descriptor
    \param outp(::OUTPLUG**) Placeholder for output plugin descriptor
    \param DecodeCallback (function ptr) Pointer to generic decode function

    This function is called by the main process in order to register the
    plugin. The function then populates the placeholders for the plugin
    types it supports, and sets all others to null. The DecodeCallback function
    is stored in case the plugin ever wants to decode anything.
 */
/****************************************************************************/
int pluginRegister(INPLUG **inp, ANPLUG **anp, PROTOPLUG **protop, OUTPLUG **outp,
		   int(*DecodeCallback)(PACKET*,PROTO)){
  if(DEBUG_ETHERNET>=DEBUG_DO_ENTRY_ALL){
    printf("Enter pluginRegister(inp(%lx),anp(%lx),protop(%lx),outp(%lx),DecodeCallback(%lx))\n", inp, anp, protop, outp, DecodeCallback);
  }

  Decode = DecodeCallback;
  *inp = (INPLUG*)0;
  *anp = (ANPLUG*)0;
  *outp = (OUTPLUG*)0;
/*   *protop = &pp; */

  if(!pp){
    (*protop) = (PROTOPLUG*)malloc_safe(sizeof(PROTOPLUG));
    if(!(*protop)){
      printf("ERR: Malloc failed when registering ethernet plugin\n");
    }
    else {
      (*protop)->name = strdup(PROTO_ETH_NAME);
      (*protop)->nProto = 1;
      (*protop)->version = 0x00010000;
      (*protop)->pProto = (PROTO*)malloc_safe(sizeof(PROTO));
      (*protop)->sProto = (char**)malloc_safe(sizeof(char*));
      if(!((*protop)->name) || !((*protop)->pProto) || !((*protop)->sProto)){
	printf("ERR: Malloc failed when registering ethernet plugin\n");
	(*protop) = (PROTOPLUG*)0;
      }
      else {
	((*protop)->pProto)[0] = PROTO_ETH_NUM;
	((*protop)->sProto)[0] = strdup_safe(PROTO_ETH_NAME);
      }
    }
    pp = *protop;
  }
  else *protop = pp;

  return 0;
}

/****************************************************************************/
/** \brief Initialise the plugin
    \param ct (int) Number of initialisation strings
    \param inits (char **) Array of ct initialisation strings
 */
/****************************************************************************/
int pluginInit(int ct, char **inits){
  if(DEBUG_ETHERNET>=DEBUG_DO_ENTRY_ALL){
    printf("Enter pluginInit(int ct(%d),char**inits(%lx))\n", ct, inits);
  }
  return 0;
}

/****************************************************************************/
/** \brief Cleanup the plugin
    \param reason (int) The reason for the cleanup.

    This call should always be made before closing the program, or
    reinitialising the plugin. It frees all the memory allocated and still held
    internally (i.e. it doesn't free memory that it has a pointer to but has
    passed elsewhere). Additionally, files are closed etc.
 */
/****************************************************************************/
int pluginCleanup(int reason){
  if(DEBUG_ETHERNET>=DEBUG_DO_ENTRY_ALL){
    printf("Enter pluginCleanup(int reason(%d))\n", reason);
  }
  
  if(pp) free_safe(pp);
  pp = (PROTOPLUG*)0;

  return 0;
}

/****************************************************************************/
/** \brief Decodes the packet for this protocol
    \param p (::PACKET*) The packet to decode
    \param protocol (::PROTO) The protocol to decode

    The plugin attempts to detect whether this protocol is the next undecoded
    protocol in the packet. If it is, then it is decoded (i.e. pointers to the
    header, footer and payload are given), and the protocol descriptor is
    appended to the packet. If the protocol isn't the next, then the decode
    callback function will be used to attempt to decode until it is found,
    or no more can be decoded.
 */
/****************************************************************************/
int pluginDecode(PACKET *p, PROTO protocol){
  if(DEBUG_ETHERNET>=DEBUG_DO_ENTRY_SOME){
    printf("Enter pluginDecode(PACKET*p(%lx),PROTO protocol(%lx))\n", p, protocol);
  }
  return 0;
}

/****************************************************************************/
/** \brief Performs a 'test' on the data
    \param p (::PACKET*) The packet to test
    \param arg (char*) The test to perform
    \param val (char*) The value to check for

    This function is used to perform 'special' tests which cannot be performed
    by just comparing a header to a value. This is esepcially common in the 
    higher layer protocols. It is also used for optional header fields and to
    check for special conditions such as valid or invalid checksums.
 */
/****************************************************************************/
int pluginTest(PACKET *p, char *arg, char *val){
  if(DEBUG_ETHERNET>=DEBUG_DO_ENTRY_ALL){
    printf("Enter pluginTest(PACKET*p(%lx),char*arg(%s), char*val(%s))\n", p, arg, val);
  }
  return 0;
}

/****************************************************************************/
/** \brief Prints a specific field
    \param p (::PACKET*) The packet to test
    \param args (char*) The test to perform
    \param print (function pointer) The function to call to print out to

    This function prints the field specified by args of packet p. It does this
    using the 'print' function. This is to allow the same function to be able
    to print to a growable buffer in memory, a pipe, stream, or file. The 
    print function should be used as printf would be used - indeed most of the
    time this will essentially be a wrapper around vfprintf or vsprintf. This
    allows the protocol plugin to decide _how_ it prints the field, for example
    an IP address can be printed in dotted form instead of just a long.
 */
/****************************************************************************/
int pluginPrint(PACKET *p, char *args,
		int(*print)(char *fmt, ...)){
  if(DEBUG_ETHERNET>=DEBUG_DO_ENTRY_ALL){
    printf("Enter pluginPrint(PACKET*p(%lx),char*args(%s),print(%lx))\n", p, args, print);
  }
  return 0;
}

/****************************************************************************/
/** \brief Translates a string description of a field and value to test into
    a policy test.
    \param p (::PROTO) Protocol
    \param val (char*) Value
    \param test (char*) Test type
    \param field (char*) Field
    \param pTest (::POLICY_TEST*) The policy test to populate

    This function allows the plugin to describe in policy form the test to 
    perform for the specified string test. The pTest function is to be 
    populated (and so must not be NULL) and it is then inserted into the
    policy.
*/
/****************************************************************************/
int pluginMkTest(PROTO p, char *val, char *test, char *field, POLICY_TEST *pTest){
  if(DEBUG_ETHERNET>=DEBUG_DO_ENTRY_ALL){
    printf("Enter pluginMkTest(PROTO p(%lx),char*val(%s),char*test(%s),char*field(%s),POLICY_TEST*pTest(%lx))\n", p, val, test, field, pTest);
  }

  return 0;
}
